@@ -1,11 +1,9 @@
-package Mojo::Promise::Role::Repeat;
+package Mojo::Promise::Role::Repeat 0.003;
 
 # ABSTRACT: Promise looping construct with break
 
 use Mojo::Base -role;
 
-our $VERSION = '0.002';
-
 sub repeat {
     my ($self, $body) = (shift, pop);
     unless (ref $self) {
@@ -49,6 +47,44 @@ sub repeat {
     return $done_p;
 };
 
+sub repeat_catch {
+    my ($self, $body) = (shift, pop);
+    unless (ref $self) {
+	$self = $self->reject(@_);
+    }
+    elsif (@_) {
+	my @values = @_;
+	$self = $self->catch( sub { (@values, @_) });
+    }
+    my $done_p = $self->clone;
+    my $break = sub {
+	$done_p->reject(@_);
+	die $self->clone;
+	# kill/abandon whatever handler we are in.
+	# Same as for repeat()
+    };
+    my $again_w;
+    $again_w = sub {
+	my $again = $again_w;
+	$self = $self->catch(
+	    sub {
+		$again->();
+		my @r;
+		@r = $body->(@_) for ($break);
+		return @r;
+	    }
+	)->then(
+	    sub {
+		$done_p->resolve(@_);
+		die $self->clone;
+	    }
+	);
+    };
+    $again_w->();
+    Scalar::Util::weaken($again_w);
+    return $done_p;
+};
+
 1;
 __END__
 
@@ -58,6 +94,10 @@ __END__
 
 Mojo::Promise::Role::Repeat - Promise looping construct with break
 
+=head1 VERSION
+
+version 0.003
+
 =head1 SYNOPSIS
 
   # stupidly complicated while loop
@@ -94,19 +134,21 @@ L<Mojo::Promise::Role::Repeat>, a role intended for L<Mojo::Promise> objects, pr
 
 =head1 METHODS
 
+In all of the following
+
+  $class   = Mojo::Promise->with_roles('+Repeat');
+  $promise = $class->new;  # or some other promise object of this class
+
 L<Mojo::Promise::Role::Repeat> supplies the following methods to the host object/class:
 
 =head2 repeat
 
-  $done = Mojo::Promise->with_roles('+Repeat')
-          ->repeat(@initial, sub {...});
-  $done = $promise
-          ->repeat(@initial, sub {...});
+  $done = $class  ->repeat(@initial, sub {...});
+  $done = $promise->repeat(@initial, sub {...});
 
 The first form is equivalent to
 
-  $done = Mojo::Promise->resolve(@initial)
-          ->then{sub {...})
+  $done = $class->resolve(@initial)
           ->then(sub {...})
           ->then(sub {...})
           # ... forever
@@ -114,7 +156,6 @@ The first form is equivalent to
 The second form is equivalent to
 
   $done = $promise->then( sub { (@initial, @_) } )
-          ->then(sub {...})
           ->then(sub {...})
           ->then(sub {...})
           # ... forever
@@ -139,7 +180,34 @@ Note that the "break" function can be used to break out of nested handlers, but
         })
     })
 
-Note that this method is B<EXPERIMENTAL> and might change without warning!
+Note that this method is B<EXPERIMENTAL> and might change without warning.
+
+=head2 repeat_catch
+
+  $done = $class  ->repeat_catch(@initial, sub {...});
+  $done = $promise->repeat_catch(@initial, sub {...});
+
+The first form is equivalent to
+
+  $done = $class->reject(@initial)
+          ->catch(sub {...})
+          ->catch(sub {...})
+          # ... forever
+
+The second form is equivalent to
+
+  $done = $promise->catch( sub { (@initial, @_) } )
+          ->catch(sub {...})
+          ->catch(sub {...})
+          # ... forever
+
+In both cases, the value returned is effectily the promise generated by the "last" C<catch> call, the effect being to invoke the handler (C<sub {...}>) repeatedly for as often as it keeps failing, each time passing the (error) values thrown from the previous iteration, and with C<$_> bound to a "break" function that, when called, does not return but instead exits the handler, abandons the loop, and rejects that final promise (C<$done>) with the arguments provided.
+
+If the "break" function is invoked with a promise passed as its first argument, the handler and loop are likewise abandoned, and the final promise awaits resolution/rejection of the passed promise.
+
+If any iteration of the handler returns normally or returns a returns a promise that is resolved, the final promise is likewise resolved with the same values.
+
+Note that this method is B<EXPERIMENTAL> and might change without warning.
 
 =head1 SEE ALSO
 
